function [U,iter,res] = GaussSeidel(A,RHS,maxit,toler)
% Gauss-Seidel iteration for AU = RHS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N = length(RHS); % problem size

% split the matrix
B = tril(A); % lower triangular part of A

D = B-A;

normRHS = norm(RHS,2);

% initialise
U = ones(N,1);

% for residual visualisation
residuals = NaN*zeros(maxit,1);

for iter=1:maxit
 U = B\(D*U+RHS);
 
 res = norm(A*U-RHS,2)/normRHS;
 
 residuals(iter) = res;
 
 if res<=toler
   break;
 end
end

if iter==maxit
  warning('Gauss-Seidel: maximal number of iterations reached')
end

if 0
  figure(202);
  plot([1:maxit]',residuals,'.');
  title('Decay of relative residual for Gauss-Seidel iteration')
end

if 1
  % spectral radius of the iteration matrix
  E = eig(full(B\D));
  rho = max(abs(E));
  disp(['Spectral radius of iteration matrix (Gauss Seidel):) ',num2str(rho)]);
  figure(302);
  plot(real(E),imag(E),'r*')
  title('Eigenvalues (Gauss Seidel)')
  grid on
end