function [U,iter,res] = Jacobi(A,RHS,maxit,toler)
% Jacobi iteration for AU = RHS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N = length(RHS); % problem size

% split the matrix
d = diag(A);
B = spdiags(d,0,N,N);

D = B-A;

normRHS = norm(RHS,2);

% initialise
U = 0*ones(N,1);

% for residual visualisation
residuals = NaN*zeros(maxit,1);

for iter=1:maxit
 U = B\(D*U+RHS);
 
 res = norm(A*U-RHS,2)/normRHS;
 
 residuals(iter) = res;
 
 if res<=toler
   break;
 end
end

if iter==maxit
  warning('Jacobi: maximal number of iterations reached')
end

if 0
  figure(201);
  plot([1:maxit]',residuals,'.');
  title('Decay of relative residual for Jacobi iteration')
end

if 1
  % spectral radius of the iteration matrix
  E = eig(full(B\D));
  rho = max(abs(E));
  disp(['Spectral radius of iteration matrix (Jacobi:) ',num2str(rho)]);
  figure(301);
  plot(real(E),imag(E),'r*')
  title('Eigenvalues (Jacobi)')
  grid on
end